/*-----------------------------------------------------------------------------
 * Copyright (c) 2009 Civic Computing Ltd.
 * All rights reserved.
 *
 * This file is part of Content Control.
 *
 * Content Control is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Content Control is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Content Control.  If not, see http://www.gnu.org/licenses/.
 *
 * Revision      $Rev: 3042 $
 * Modified by   $Author: keith $
 * Modified on   $Date: 2010-08-17 19:50:42 +0300 (Tue, 17 Aug 2010) $
 *
 * Changes: see subversion log.
 *-----------------------------------------------------------------------------
 */
package ccc.api.exceptions;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import ccc.api.core.Failure;
import ccc.api.types.DBC;


/**
 * An exception representing the failure of a CCC command.
 *
 * @author Civic Computing Ltd.
 */
public class CCException
    extends
        RuntimeException {

    /** MESSAGE : String. */
    public static final String MESSAGE = "message";

    private UUID                _id     = UUID.randomUUID();
    private Map<String, String> _params = new HashMap<String, String>();


    /** Constructor. */
    public CCException() { this(null, null, new HashMap<String, String>()); }


    /**
     * Constructor.
     *
     * @param message Description of the exception.
     * @param cause   Cause of the exception.
     * @param params  Further details describing the exception.
     */
    protected CCException(final String message,
                          final Throwable cause,
                          final Map<String, String> params) {
        super(message, cause);
        _params.putAll(params);
        _params.put(MESSAGE, ""+message); // NPE has NULL message :(
        _params.put("cause", (null==cause) ? null : ""+cause.getMessage());
    }


    /**
     * Constructor.
     *
     * @param message Description of the exception.
     */
    public CCException(final String message) {
        this(
            message,
            null,
            new HashMap<String, String>());
    }


    /**
     * Constructor.
     *
     * @param message Description of the exception.
     * @param cause   Cause of the exception.
     */
    public CCException(final String message,
                       final Throwable cause) {
        this(
            message,
            cause,
            new HashMap<String, String>());
    }


    /**
     * Accessor.
     *
     * @return The failure.
     */
    public Failure getFailure() {
        return new Failure(_id, getClass().getName(), _params);
    }


    /**
     * Accessor.
     *
     * @param key The parameter's key.
     *
     * @return The parameter's value.
     */
    protected String getParam(final String key) {
        return _params.get(key);
    }


    /**
     * Mutator.
     *
     * @param id The id to set.
     */
    public void setId(final UUID id) {
        _id = DBC.require().notNull(id);
    }


    /**
     * Mutator.
     *
     * @param params The parameters to set.
     */
    public void setParams(final Map<String, String> params) {
        DBC.require().notNull(params);
        _params.clear();
        _params.putAll(params);
    }


    /** {@inheritDoc} */
    @Override
    public String getMessage() {
        final String message = _params.get(MESSAGE);
        if (null!=message) { return message; }
        return super.getMessage();
    }


    /**
     * Add a parameter to the exception.
     *
     * @param key   The paramater's key.
     * @param value The parameter's value.
     */
    protected void addParam(final String key, final String value) {
        _params.put(key, value);
    }
}
